#!/bin/bash

#set default values to be used when not passed as command line options 

# the default value used when --loopCount is not specified
defaultLoopCount=3

# the default value used when --timeLapse is not specified
defaultTimeLapse=0.40

# the default value used when --outputSize is not specified
defaultOutputSize=1280x720

# the default value used when --frameRate is not specified
defaultFrameRate=30

# the default value used when --compression is not specified
defaultCompression=28

# the default value used when --preset is not specified
defaultPreset=superfast

# the default value used when --preset is not specified
defaultAudio=noAudio


#########################################
##### DO NOT MODIFY BELOW THIS LINE #####
#########################################
usage () {
    echo 'usage: <path_to_this_file> --inputFile <path_to_input_file> --outputFile <path_to_output_file> [ --outputSize <1280x720 etc...> ] [ --frameRate <frames_per_sec> ] [ --compression <1 to FLOAT_MAX> ]'
}
die () {
    echo FATAL ERROR: $@ >&2
    usage
    exit 1;
}
#set the PATH to include /usr/local/bin
if test "$PATH" = "${PATH//\/usr\/local\/bin}"; then
  export PATH=/usr/local/bin:$PATH
fi
declare -i squareOutput=0

# parse arguments for recognized flags
while (( $# > 0 )) ; do
 echo $1 = $2
 case "$1" in
    -i|--inputFile)
    inputFile="$2"
    shift # past argument
    ;;
    -o|--outputFile)
    outputFile="$2"
    shift # past argument
    ;;
    -v|--overlayFile)
    overlayFile="$2"
    shift # past argument
    ;;
    -l|--loopCount)
    loopCount="$2"
    shift # past argument
    ;;
    -t|--timeLapse)
    timeLapse="$2"
    shift # past argument
    ;;
    -p|--preset)
    preset="$2"
    shift # gobble up $2 argument
    ;;
    -s|--outputSize)
    outputSize="$2"
    shift # gobble up $2 argument
    ;;
    -r|--frameRate)
    frameRate="$2"
    shift # gobble up $2 argument
    ;;
    -c|--compression)
    compression="$2"
    shift # gobble up $2 argument
    ;;
    -q|--square)
#    squareOutput=1
    skwear="-vf \"crop='if(gt(iw,ih),ih,iw)':'if(gt(iw,ih),ih,iw)':'if(gt(iw,ih),(iw-ih)/2,(ih-iw)/2)':0\""
    skwear="-vf \"crop='if(gt(iw,ih),ih,iw)':'if(gt(iw,ih),ih,iw)'\""
    ;;
    -w|--withAudio)
        audioFormat="-c:a libfdk_aac -ac 2 -ar 48000 -af aresample=async=2"
    ;;
    *)
    # unknown option
    echo unrecognized option $1
    break
    ;;
 esac
 shift # past argument or value
done

#check for fixed arguments usage
if test -z "${inputFile}" && (($# == 5)); then
 echo using fixed arguments
 #arg 1 is path to input file
 inputFile="${1}"

 #arg 2 is path to input file
 outputFile="${2}"

 #arg 3 is overlay file path
 overlayFile="${3}"

 #arg 4 is loop count (a loop is forward and reverse)
 loopCount="${4:-3}"

 #arg 5 is pts (timeLapse)
 timeLapse="${5:-0.4}"

fi

#error check the input arguments

# verify input file
test -r "${inputFile}" || die unreadable input file "${inputFile}"
#verify output path
test "${outputFile}" || die you must specify an output file path
#verify overlay file type
if test "${overlayFile}"; then
 overlayFileType=$(file  "${overlayFile}")
 test "${overlayFileType}" != "${overlayFileType//PNG image data}" || die overlay files must be PNG type
fi
#validate loopCount
if test -z "${loopCount}" ;then
 loopCount=$defaultLoopCount
 echo "--loopCount = $loopCount (using default value)"
fi
#validate timeLapse
if test -z "${timeLapse}" ;then
     timeLapse=$defaultTimeLapse
     echo "--timeLapse = $timeLapse (using default value)"
fi
#validate outputSize
if test -z "${outputSize}" ;then
    outputSize=$defaultOutputSize
    echo "--outputSize = $outputSize (using default value)"
fi
#validate frameRate
if test -z "${frameRate}" ;then
    frameRate=$defaultFrameRate
    echo "--frameRate = $frameRate (using default value)"
fi
#validate compression
if test -z "${compression}" ;then
    compression=$defaultCompression
    echo "--compression = $compression (using default value)"
fi
#validate preset
if test -z "${preset}" ;then
    preset=$defaultPreset
    echo "--preset = $preset (using default value)"
fi
#0. may need to encode to specs first

inputFileBase="${inputFile//.m??}"

#set file extension automatically
fileExt="${inputFile//${inputFileBase}}"

#1. remove audio (and downconvert video)
workFile1="${inputFileBase}.noAudio${fileExt}"
eval ffmpeg -nostats -loglevel error -i "${inputFile}" -pix_fmt yuv420p -preset "${preset}" -r "${frameRate}" -crf "${compression}" "${skwear}" -c:v libx264 "${audioFormat:--an}" -y "${workFile1}"

# skip the rest for now
#if true; then
#    mv "${workFile1}" "${outputFile}"
#else

#2. time lapse
if((loopCount > 0)); then
    workFile2="${inputFileBase}.timeLapse${fileExt}"
    ffmpeg -nostats -loglevel error -i "${workFile1}" -filter_complex "[0:v]setpts=${timeLapse:-0.25}*PTS[v]" -map "[v]" -y "${workFile2}"
else
    workFile2="${workFile1}"
fi

#3. make it square
if((squareOutput)); then
    echo converting output to square crop
    workFile3="${inputFileBase}.square${fileExt}"
    ffmpeg -nostats -loglevel error -i "${workFile2}"  -vf "crop='if(gt(iw,ih),ih,iw)':'if(gt(iw,ih),ih,iw)'" -y "${workFile3}"
else
    workFile3="${workFile2}"
fi
    #4. overlay

if test -r "${overlayFile}"; then
    workFile4="${inputFileBase}.overlay${fileExt}"
#    ffmpeg -nostats -loglevel error -i "${workFile3}" -i "${overlayFile}" -filter_complex "[0:v]overlay=:0[v]" -map "[v]" -y "${workFile4}"
ffmpeg -nostats -loglevel error -i "${workFile3}" -i "${overlayFile}" -filter_complex "overlay" -y "${workFile4}"
else
    workFile4="${workFile3}"
fi


    #fwd1-rev1-fwd2-rev2-fwd3-rev3…fwdN-revN
if((loopCount > 0)); then
    #5.reverse
    workFile5="${inputFileBase}.reverse${fileExt}"
    ffmpeg -nostats -loglevel error -i "${workFile4}" -filter_complex "[0:v]reverse[v]" -map "[v]" -y "${workFile5}"
    #6. concat it n-times
    workFile6="${inputFileBase}.looper${fileExt}"
    ffmpeg -nostats -loglevel error -f concat -safe 0 -i <(for (( ((i=0)) ; ((i<loopCount)) ; ((i++)) )) ; do printf "file %s\n" "${workFile4}" "${workFile5}" ; done) -c copy -y "${workFile6}"
else
  workFile6="${workFile4}"
fi

# move product to output path
cp "${workFile6}" "${outputFile}"

rm -rf "${workFile1}"
rm -rf "${workFile2}"
rm -rf "${workFile3}"
rm -rf "${workFile4}"
rm -rf "${workFile5}"
rm -rf "${workFile6}"
rm -rf "${workFile7}"

